%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Modification of RPCA from below %%%%%%%%%%%%%%%%%%
%%%%%%%% https://github.com/dlaptev/RobustPCA %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [L, S] = SLdecompose(X, opts)

if ~isfield(opts, 'lambdaR')
   lambdaR = 0.01*1/sqrt(max(size(X)));   
else
   lambdaR = opts.lambdaR;
end
if ~isfield(opts, 'maxIter')
   maxIter = 20;
else 
    maxIter = opts.maxIter;
end
if ~opts.quickMode
    lambda = 1*lambdaR/3;
    mu = 10*lambdaR;
    [M, N] = size(X);
    normX = norm(X, 'fro');  

    L = zeros(M, N, 'single');
    S = zeros(M, N, 'single');
    Y = zeros(M, N, 'single');

    for iter = 1:maxIter 

        if isfield(opts, 'doMonitor')
            if opts.doMonitor
%                 sReshape = reshape( S(:,opts.monitorFrame), [opts.imgResolution(1) opts.imgResolution(2)]);
%                 xReshape = reshape( X(:,opts.monitorFrame), [opts.imgResolution(1) opts.imgResolution(2)]);
%                 lReshape = reshape( L(:,opts.monitorFrame), [opts.imgResolution(1) opts.imgResolution(2)]);
                sReshape = reshape( S(opts.monitorFrame,:), [opts.imgResolution(1) opts.imgResolution(2)]);
                xReshape = reshape( X(opts.monitorFrame,:), [opts.imgResolution(1) opts.imgResolution(2)]);
                lReshape = reshape( L(opts.monitorFrame,:), [opts.imgResolution(1) opts.imgResolution(2)]);            
                sShow = sReshape( opts.centerRegionY , opts.centerRegionX );
                xShow = xReshape( opts.centerRegionY , opts.centerRegionX );
                lShow = lReshape( opts.centerRegionY , opts.centerRegionX );

                maxX = max(xShow(:));
                figure(1); imshow( [opts.sGain*sShow'/maxX, lShow'/maxX, xShow'/maxX ]); drawnow;
            end
        end


        tic;

        L = Do(1/mu, X - S + (1/mu)*Y);
        L = max(L,0);
        S = So(lambda/mu, X - L + (1/mu)*Y);
        S = max(S,0);

        Z = X - L - S;
        Y = Y + mu*Z;

        err = norm(Z, 'fro') / normX;

        ttime = toc;
        disp([num2str(iter) '|' num2str(maxIter) 'th iteration. Error: ' num2str(err) '. It took ' num2str(ttime) ' secs.']);      

    end
else
    L = repmat(min(X,[],2), 1, size(X,2));
    Lfirst = L(:,1);
    S = X - L;
end

end

function r = Do(tau, X)
    [U, S, V] = svd(X, 'econ');
    r = U*So(tau, S)*V';
end

function r = So(tau, X)
    r = sign(X) .* max(abs(X) - tau, 0);
end