%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Sparse Decomposition LFM (SDLFM).
%% Decomposition based on RPCA from https://github.com/dlaptev/RobustPCA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
clc;
addpath('./UtilMIP/');
addpath('./UtilXLFM/');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
currentSlice = single(imread('./DATA/SDLFM_testData.tif', 1));

figure; imshow(currentSlice/max(currentSlice(:)));
currentSlice = currentSlice(:, (1+100:end-100));
imgResolution = size(currentSlice);
tlength = 10;

X = zeros( imgResolution(1)*imgResolution(2), tlength, 'single');
for ti=1:tlength
    currentSlice = single(imread('./DATA/SDLFM_testData.tif', ti));
    currentSlice = currentSlice(:, (1+100:end-100));
    X(:,ti) = currentSlice(:);
end

centerX = (905:1455);
centerY = (910:1220);
figure; imshow(currentSlice(centerY, centerX)/max(currentSlice(:)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
opts.lambdaR = 0.05*1/sqrt(max(size(X)));
opts.maxIter = 20;
opts.doMonitor = 0;
opts.centerRegionX = centerX;
opts.centerRegionY = centerY;
opts.monitorFrame = 6;
opts.imgResolution = imgResolution;
opts.sGain = 2;
opts.quickMode = false;
[L, S] = SLdecompose(X', opts);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('./PSF/SDLFM_PSF.mat');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NxyExt=164;
Nxy=size(PSFG1,1)+NxyExt*2;
Nz=size(PSFG1,3);
ROISize=400;

zRangePSF = (1+40:200-10);
PSF_A=padarray(PSFG1( :, (1+100:end-100),zRangePSF),[100+NxyExt NxyExt 0],0,'both');
PSF_B=padarray(PSFG2( :, (1+100:end-100),zRangePSF),[100+NxyExt NxyExt 0],0,'both');

gpuDevice()
PSF_Agpu=gpuArray(PSF_A);
PSF_Bgpu=gpuArray(PSF_B);
backwardFUN = @(projection) backwardXLFM( gpuArray(projection) , PSF_Agpu, PSF_Bgpu, RatioAB, ROISize);
forwardFUN = @(Xguess) forwardXLFM(Xguess, PSF_Agpu, PSF_Bgpu, RatioAB, ROISize);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ItN = 30;
centerXR = [150:650];
centerYR = (230:570);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
deconMethod = 2;
Xall = zeros( length(centerYR), length(centerXR), length(zRangePSF), tlength, 'uint16');

for t=1:tlength,    
    disp(['Running reconstruction of ' num2str(t) ' | ' num2str(tlength) ' th frame...']);
    
    ImgExp = padarray(reshape( S(t,:) , [imgResolution(1) imgResolution(2)]) ,[100+NxyExt NxyExt 0], 0, 'both');
    Htf = backwardFUN(ImgExp);
    Xguess = Htf;
    XguessShow = flip(Xguess(centerYR, centerXR,:),3);
    figure(2); imshowMIPorth( XguessShow/max(XguessShow(:)), 2.5/1.6, 1, 25);
    
    for i=1:ItN,
        tic;
        switch deconMethod
            case 1
                Xguess = deconvRLonestep( forwardFUN, backwardFUN, ImgExp, Xguess);
                
            case 2
                lambdaD = 0.05;
                Xguess = deconvSRLonestep( forwardFUN, backwardFUN, ImgExp, Xguess, lambdaD);                
               
                
        end
        ttime = toc;        
        XguessShow = flip(Xguess(centerYR, centerXR,:),3);           
        figure(2); imshowMIPorth( XguessShow/max(XguessShow(:)), 2.5/1.6, 1, 25);
        disp(['  iter ' num2str(i) ' | ' num2str(ItN) ', took ' num2str(ttime) ' secs']);
    end
    
    if t==1,
        MV3Dgain = 0.25/max(XguessShow(:));        
    end
    Xvolume = uint16(round(65535*MV3Dgain*XguessShow));
    Xall(:,:,:,t) = Xvolume;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

